/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
#ifndef _LKL_LINUX_POSIX_TYPES_H
#define _LKL_LINUX_POSIX_TYPES_H

#include <lkl/linux/stddef.h>

/*
 * This allows for 1024 file descriptors: if LKL_NR_OPEN is ever grown
 * beyond that you'll have to change this too. But 1024 fd's seem to be
 * enough even for such "real" unices like OSF/1, so hopefully this is
 * one limit that doesn't have to be changed [again].
 *
 * Note that POSIX wants the FD_CLEAR(fd,fdsetp) defines to be in
 * <sys/time.h> (and thus <linux/time.h>) - but this is a more logical
 * place for them. Solved by having dummy defines in <sys/time.h>.
 */

/*
 * This macro may have been defined in <gnu/types.h>. But we always
 * use the one here.
 */
#undef __LKL__FD_SETSIZE
#define __LKL__FD_SETSIZE	1024

typedef struct {
	unsigned long fds_bits[__LKL__FD_SETSIZE / (8 * sizeof(long))];
} __lkl__kernel_fd_set;

/* Type of a signal handler.  */
typedef void (*__lkl__kernel_sighandler_t)(int);

/* Type of a SYSV IPC key.  */
typedef int __lkl__kernel_key_t;
typedef int __lkl__kernel_mqd_t;

#include <lkl/asm/posix_types.h>

#endif /* _LKL_LINUX_POSIX_TYPES_H */
